package org.ajax4jsf.builder.mojo;

import java.io.File;
import java.io.FileFilter;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.velocity.VelocityContext;

/**
 * @goal add-skin
 * @author Maksim Kaszynski
 *
 */
public class CreateSkinMojo extends AbstractCDKMojo {

	private static final String TEMPLATES_PREFIX = "/skin/";

	private static final String XCSS_TEMPLATE = TEMPLATES_PREFIX + "skin.xcss";

	private static final String PROPERTIES_TEMPLATE = TEMPLATES_PREFIX + "skin.properties";

	/**
	 * @parameter expression="${name}"
	 * @required
	 */
	private String name;

	public void execute() throws MojoExecutionException, MojoFailureException {

		getLog().info("Generating Skin");
		name = name.replaceAll("\\s", "_").toLowerCase();

		getLog().debug("Skin name is supposed to be " + name);

		int lastIndexOfDot = name.lastIndexOf('.');
		String packageName;
		
		if (lastIndexOfDot > 0) {
			packageName = name;//name.substring(0, lastIndexOfDot);
			name = name.substring(lastIndexOfDot + 1);
		} else {
			packageName = project.getGroupId() + "." + name;
		}



		getLog().debug("Package detected " + packageName);

		File resourceDir = new File(project.getBasedir(), "src/main/resources");
		getLog().debug("Resources directory is supposed to be" + resourceDir);
		checkAndCreateDirectory(resourceDir);

		String packageFolder = packageName.replace('.', '/');
		getLog().debug("Relative path to package is " + packageFolder);

		File skinDir = new File(resourceDir, packageFolder);
		checkAndCreateDirectory(skinDir);
		
		
		getLog().debug("basic XCSS is supposed to be dropped into " + skinDir);

		File[] skinFiles = skinDir.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
				return pathname.isFile() && pathname.getName().endsWith(".xcss");
			}
		});
		

		File baseXcss = new File(skinDir, "basic.xcss");

		File metaInfSkinsDiir = new File(resourceDir, "META-INF/skins");
		getLog().debug("Property file and XCSS will be dropped into  " + skinDir);

		checkAndCreateDirectory(metaInfSkinsDiir);

		File xcssFile = new File(metaInfSkinsDiir, name + ".xcss");
		File propertyFile = new File(metaInfSkinsDiir, name + ".properties");


		VelocityContext context = new VelocityContext();
		context.put("baseXcss", baseXcss);
		context.put("xcssFiles", skinFiles);
		context.put("packagePath", packageFolder);
		context.put("skinDir", skinDir);
		context.put("xcssFile", xcssFile);
		context.put("propertyFile", propertyFile);
		context.put("name", name);

		try {
			getLog().debug("Creating Skin property file " + propertyFile);
			writeParsedTemplate(PROPERTIES_TEMPLATE, context, propertyFile);
			getLog().debug("Creating XCSS file " + xcssFile);
			writeParsedTemplate(XCSS_TEMPLATE, context, xcssFile);

		} catch (Exception e) {
			throw new MojoExecutionException("An exception occured while peerforming component generatuion", e);
		}

		getLog().info("Generating Skin successful");

	}


	private void checkAndCreateDirectory(File directory) throws MojoExecutionException {
		getLog().debug("Checking directory " + directory + " for existence");
		if (!directory.exists()) {
			directory.mkdirs();
			getLog().debug("Directory " + directory + " created");
		}
		if (!directory.isDirectory()) {
			throw new MojoExecutionException("Directory " + directory.getAbsolutePath() + " is not a Directory");
		}
	}

}
